#!/usr/bin/env perl

use Data::Dumper;

use NetSNMP::agent (':all');
use NetSNMP::ASN qw(ASN_OCTET_STR ASN_INTEGER);

my $oid_prefix = ".1.3.9999.51";

# Configure a timer that will poll status for each zpool
my %oid_data;
my $first_oid = "";
my $last_oid = "";

sub set_zpool_oid_value {
  my ($zpool_index, $param_index, $value, $type) = @_;

  my $this_oid = new NetSNMP::OID("$oid_prefix.$zpool_index.$param_index");

  if (!$first_oid) {
    $first_oid = $this_oid;
  }

  $oid_data{$this_oid}{'value'} = $value;
  $oid_data{$this_oid}{'type'} = $type;
  $oid_data{$this_oid}{'next'} = "";

  # Set the last item's 'next' point to this OID
  if ($last_oid) {
    $oid_data{$last_oid}{'next'} = $this_oid;
  }
  $last_oid = $this_oid;
}

sub force_refresh_zpool_data {
  my $index = 0;

  my $zpool_payload = `zpool list -Hpo name,health`;
  my @zpools = split('\n', $zpool_payload);

  foreach my $zpool (@zpools) {
    my @zpool_data = split('\t', $zpool);

    # Set general information
    set_zpool_oid_value($index, 0, $zpool_data[0], ASN_OCTET_STR);
    set_zpool_oid_value($index, 1, uc $zpool_data[1], ASN_OCTET_STR);

    $index++;
  }
}

# Don't refresh data every time a request comes in.  Let 3 minutes expire
#   before data is refreshed
my $last_update_time = 0;
sub refresh_zpool_data {
  my $current_time = time();

  if ($current_time > ($last_update_time + 60 * 3)) {
    force_refresh_zpool_data();
    $last_update_time = $current_time;
  }
}

# Handle OID requests
sub zpool_status_handler {
  my ($handler, $registration_info, $request_info, $requests) = @_;

  refresh_zpool_data();

  my $request;
  for($request = $requests; $request; $request = $request->next()) {
    my $oid = $request->getOID();

    if ($request_info->getMode() == MODE_GET) {
      if (exists($oid_data{$oid})) {
        my $oid_value = $oid_data{$oid};
        $request->setValue($oid_value->{'type'}, $oid_value->{'value'});
      }
    }

    elsif ($request_info->getMode() == MODE_GETNEXT) {
      if (exists($oid_data{$oid})) {
        $next_oid = $oid_data{$oid}->{'next'};

        if (exists($oid_data{$next_oid})) {
          my $oid_value = $oid_data{$next_oid};

          $request->setOID($next_oid);
          $request->setValue($oid_value->{'type'}, $oid_value->{'value'});
        }
      }

      elsif ($oid < $first_oid) {
        my $oid_value = $oid_data{$first_oid};

        $request->setOID($first_oid);
        $request->setValue($oid_value->{'type'}, $oid_value->{'value'});
      }
    }
  }
}

#refresh_zpool_data();

my $zpool_status_agent = new NetSNMP::agent();
$zpool_status_agent->register("zpool_status", $oid_prefix, \&zpool_status_handler);
